var frameSelection = null;
var requestUrl = null;

chrome.runtime.onMessage.addListener(
  function(request, sender, sendResponse) {
    if (null != request && null != request.frSelection && request.frSelection.startsWith('*selection-frame-container*:')) {
      frameSelection = request.frSelection.replace('*selection-frame-container*:', '').trim();
      requestUrl = request.tabUrl;
      if (frameSelection && frameSelection != '') {
        frameSelection = frameSelection.replace(/[\n\r]/g, ' ');
      } else {
        frameSelection = null;
      }

    }
  });

function getPageEntryContent(callback) {
  chrome.tabs.query({
    active: true,
    currentWindow: true
  }, function(tabs) {
    var selectedTab = tabs[0];
    chrome.tabs.executeScript(selectedTab.id, {
        code: "document.body.innerText"
      },
      function(results) {
        console.log("content info: " + JSON.stringify(results));
        if (results && results[0]) {
          content = results[0].replace(/[\n\r]/g, ' ').trim();
          if (content) {
            callback(content);
          } else {
            callback('');
          }
        } else {
          callback(' ');
        }
      }
    );
  });
}


function getSelectionContent(callback) {
  chrome.runtime.getPackageDirectoryEntry(function(root) {
    root.getFile("config.js", {}, function(fileEntry) {
      fileEntry.file(function(file) {
        var reader = new FileReader();
        reader.onloadend = function(e) {
          console.log(e);
          // contents are in this.result
        };
        reader.readAsText(file);
      }, errorHandler);
    }, errorHandler);
  });
  chrome.tabs.query({
    active: true,
    currentWindow: true
  }, function(tabs) {
    var selectedTab = tabs[0];
    chrome.tabs.executeScript(selectedTab.id, {
        code: captiGetSelectionText.toString() + ' captiGetSelectionText()'
      },
      function(results) {
        console.log("selected content info: " + JSON.stringify(results));
        if (results && results[0]) {
          callback(results[0]);
        } else {
          var temp_url = new URL(selectedTab.url);
          if (temp_url.origin === requestUrl) {
            callback(frameSelection);
          } else {
            callback(null);
          }
        }
      }
    );
  });
}

function captiGetSelectionText() {
  var selectionContentText = null;
  if (window.getSelection && window.getSelection().toString()) {
    selectionContentText = window.getSelection().toString();
  } else {
    function convertToSubIndex(selectObj) {
      coff = Math.round(selectObj.selectionWidth * 100 / selectObj.lineWidth);
      return Math.round(selectObj.wordCount * coff / 100);
    }

    function getGSlideSelection() {
      var all_selection = null;
      var query = " rect[shape-rendering='crispEdges'][fill='#000']";
      var arr_rect = document.querySelectorAll("g[id^='editor-']" + query); // get inactive selection rect elements
      if (0 == arr_rect.length) {
        arr_rect = document.querySelectorAll("g[id^='speakernotes-']" + query); // get inactive selection speak notes
      }

      if (0 == arr_rect.length) {
        arr_rect = document.querySelectorAll('.sketchy-text-selection-overlay'); // get active selection
      }

      if (0 != arr_rect.length) {
        all_selection = [];
      }

      arr_rect.forEach(function(rect_el) {
        var g_element = rect_el.parentElement; // back to up level
        var content_text_el = g_element.parentElement.nextElementSibling.firstElementChild; // go to text element
        var temp_selection_list = [];
        content_text_el.childNodes.forEach(function(word_el) {
          temp_selection_list.push(word_el.textContent);
        });
        var temp_selection_str = temp_selection_list.join(" ") // get all text from line

        var selection_bond_obj = rect_el.getBoundingClientRect();
        var line_bond_obj = content_text_el.getBoundingClientRect();
        var diff_x = selection_bond_obj.x - line_bond_obj.x; // calculate diff between x of selection and x of line

        var start_pos = null;
        if (0 != diff_x) {
          start_pos = convertToSubIndex({
            selectionWidth: diff_x,
            lineWidth: line_bond_obj.width,
            wordCount: temp_selection_str.length
          });
        } else {
          start_pos = 0;
        }

        var end_pos = null;
        if (selection_bond_obj.width < line_bond_obj.width) {
          end_pos = convertToSubIndex({
            selectionWidth: selection_bond_obj.width,
            lineWidth: line_bond_obj.width,
            wordCount: temp_selection_str.length
          }) + start_pos;
        } else {
          end_pos = temp_selection_str.length;
        }

        all_selection.push(temp_selection_str.slice(start_pos, end_pos));
      });

      return (null != all_selection ? all_selection.join(" ") : null);
    }


    function getGDocSelection() {
      window.gDocSelection = [];
      var selectionElements = document.querySelectorAll('.kix-selection-overlay,.kix-selection-overlay-mac');
      if (selectionElements) {
        for (i = 0; i < selectionElements.length; i++) {
          var childnode = selectionElements[i].parentNode;
          var wordGenNodes = childnode.querySelectorAll('.kix-wordhtmlgenerator-word-node');
          for (j = 0; j < wordGenNodes.length; j++) {
            selectionW = selectionElements[i].getBoundingClientRect().width - 5;
            lineW = wordGenNodes[j].getBoundingClientRect().width;
            wordL = wordGenNodes[j].textContent.length;
            window.gDocSelection.push(wordGenNodes[j].textContent.trim().slice(0, convertToSubIndex({
              selectionWidth: selectionW,
              lineWidth: lineW,
              wordCount: wordL
            })));
          }
        }
      }

      var result = (window.gDocSelection.length != 0 ? window.gDocSelection.join(" ") : null);
      window.gDocSelection = null;
      return result;
    }

    if (!selectionContentText) {
      selectionContentText = getGDocSelection();
    }

    if (!selectionContentText) {
      selectionContentText = getGSlideSelection();
    }
  }

  if (selectionContentText) {
    return selectionContentText.replace(/[\n\r]/g, ' ').trim();
  } else {
    return null;
  }
}