// functions hasClass, addClass, removeClass

Element.prototype.hasClass = function(className) {
  return new RegExp(' ' + className + ' ').test(' ' + this.className + ' ');
}

Element.prototype.addClass = function(className) {
  if (!this.hasClass(className)) {
    this.className += ' ' + className;
  }
  return this;
};

Element.prototype.removeClass = function(className) {
  var newClass = ' ' + this.className.replace(/[\t\r\n]/g, ' ') + ' ';
  if (this.hasClass(className)) {
    while (newClass.indexOf(' ' + className + ' ') >= 0) {
      newClass = newClass.replace(' ' + className + ' ', ' ');
    }
    this.className = newClass.replace(/^\s+|\s+$/g, ' ');
  }
  return this;
};

// Create context menu

var ticking = false,
  menu_container = document.createElement('div');
menu_container.id = 'cap-small-context-menu';
if ( null != document.body ) {
  document.body.appendChild(menu_container);
}

// Functions for context menu

document.onmousedown = function(ev) {
  if (ev.target.className === 'capti-item-parent-text') {
    ev.target.click();
  }
}

document.onmouseup = function(ev) {
  chrome.runtime.sendMessage({
    'option_method': 'GET'
  }, function(results) {
    if (results.captiAllowMenu === 'true') {
      if (window.getSelection && document.createRange && typeof document.createRange().getBoundingClientRect != "undefined" && window.name != 'capti-iframe') {
        var sel = window.getSelection(),
          h = Math.max(document.documentElement.clientHeight, window.innerHeight || 0);
        if (sel.isCollapsed) {
          menu_container.removeClass('active');
        } else {
          if (sel.rangeCount > 0 && sel.toString() && sel.type == 'Range') {
            var rect = sel.getRangeAt(0).getBoundingClientRect();
            x = rect.left + rect.width / 2;
            if (rect.top + rect.height + 50 > h) {
              menu_container.addClass('top');
              y = rect.top - 50;
            } else {
              if (menu_container.hasClass('top')) menu_container.removeClass('top');
              y = rect.top + rect.height + 10;
            }
            makeSmallMenu(x, y, sel);
          } else {
            menu_container.removeClass('active');
          }
        }
      } else {
        menu_container.removeClass('active');
      }
    }
  });
}

document.onselectionchange = function(ev) {
  var sel = window.getSelection();
  if (sel.type == 'None') menu_container.removeClass('active');
}

document.onkeyup = function(ev) {
  menu_container.removeClass('active');
}

window.addEventListener('scroll', function(e) {
  if (!ticking) {
    window.requestAnimationFrame(function() {
      menu_container.removeClass('active');
      ticking = false;
    });
    ticking = true;
  }
});

function makeSmallMenu(x, y, selection) {
  var sel_text = selection.toString();
  menu_container.style.left = x + 'px';
  menu_container.style.top = y + 'px';
  menu_container.innerHTML = "";

  var selectionObj = null;
  if (selection && selection.rangeCount > 0) {
    var range = selection.getRangeAt(0);
    selectionObj = {};
    selectionObj["nodeValue"] = range.commonAncestorContainer.nodeValue;
    selectionObj["startOffset"] = range.startOffset;
    selectionObj["endOffset"] = range.endOffset;
    selectionObj["parentElement"] = range.commonAncestorContainer.parentElement.outerHTML;
    selectionObj["selectionValue"] = selection.toString();
  }

  menu_container.appendChild(makeTextMenuItem('play-toolbox', 'Play (Alt + Q)', true, sel_text, range));
  menu_container.appendChild(makeTextMenuItem('view-toolbox', 'View (Alt + X)', false, sel_text, null));
  menu_container.appendChild(makeDictMenuItem('define-toolbox', 'Define', true, selectionObj));
  menu_container.appendChild(makeDictMenuItem('transition-toolbox', 'Translate', false, selectionObj));
//  menu_container.appendChild(makePlSelectMenuItem('play-toolbox', 'Play Selection', sel_text, range));
  menu_container.addClass('active');
}

function makePlSelectMenuItem(className, title, sel_text, range) {
  return makeMenuItem(className, title,
    function(ev) {
      sel_text = sel_text.replace(/(\r\n|\n|\r)/gm,' ');
      var msg = {
        external_menu_message: '*external-menu-item-view-select*:'  + sel_text,
        is_playing : true
      };

      console.log(range);

      range.startContainer.parentElement.dataset.captiStartHEl = '';
      setStartHilglightElement();

      chrome.runtime.sendMessage(msg);
    });
}

function makeDictMenuItem(className, title, isDefinition, selectionObj) {
  return makeMenuItem(className, title,
    function(ev) {

      var msg = {
        external_menu_message: '*external-menu-item-dict*',
        isDefinition: isDefinition,
        selectionObj: selectionObj
      };

      chrome.runtime.sendMessage(msg);
    });
}

function makeTextMenuItem(className, title, isPlaying, sel_text, range) {
  return makeMenuItem(className, title,
    function(ev) {
      var msg = {
        external_menu_message: '*external-menu-item-view-select*:' + sel_text
      };
      if (isPlaying) {
        msg['is_playing'] = true
        //        range.startContainer.parentElement.dataset.captiStartHEl = '';
        //        setStartHilglightElement();
      }

      chrome.runtime.sendMessage(msg);
    });
}

function makeMenuItem(iconName, text, handler) {
  var menu_item_parent = document.createElement('div');
  menu_item_parent.className = 'capti-item-parent';
  menu_item_parent.onclick = handler;

  var iconContainer = document.createElement('div');
  iconContainer.className = 'capti-icon capti-' + iconName;
  iconContainer.setAttribute('title', text);

  //var itemTitle = document.createElement('div');
  //itemTitle.className = 'capti-item-parent-text';
  //itemTitle.innerText = text;

  menu_item_parent.appendChild(iconContainer);
  //menu_item_parent.appendChild(itemTitle);
  return menu_item_parent;
}
