var bg = chrome.extension.getBackgroundPage();

var tabs = {};

chrome.tabs.onCreated.addListener(function(tab) {
  setDefaultTabCong(tab.id);
});

chrome.tabs.onUpdated.addListener(function(tabId, changeInfo, tab) {
  //    console.log('tabChanged = ' + JSON.stringify(changeInfo));
  //    console.log('tab = ' + JSON.stringify(tab));

  if (changeInfo.status == 'complete') {
    let modes = tabs[tabId];
    if (modes) {
      let mode = modes.get("current_mode");
      let options = modes.get(mode);
      if (options && options.get('wasOpenedToolbox')) {
        bg.getToolboxState(function(isShowed) {
          if (!isShowed) {
            bg.showToolBox(null, false);
          }
        });
      }
    }
  }

});

chrome.tabs.onRemoved.addListener(function(tabId) {
  let l_modes = tabs[tabId];
  if (l_modes) {
    l_modes.clear();
  }

  delete tabs[tabId];
});


chrome.runtime.onMessage.addListener(function(request, sender, sendResponse) {
  console.log('request = ' + JSON.stringify(request));
  console.log('sender = ' + JSON.stringify(sender));

  let tabId = sender.tab.id;

  if ('option-toolbox' === request.type) {

    let modes = tabs[tabId];
    if (!modes) {
      modes = setDefaultTabCong(tabId);
    }

    let mode = modes.get("current_mode");
    let selectedMode = null;
    if (request.mode) {
      if (mode != request.mode) {
        modes.set("current_mode", request.mode);
      }
      selectedMode = request.mode;
    } else {
      selectedMode = mode;
    }

    let options = modes.get(selectedMode);
    if (undefined === options) {
      options = new Map();
      modes.set(mode, options);
    }

    let t_options = request.toolbox_options;
    if (t_options) {
      let optionMap = objToMap(t_options);
      optionMap.forEach((value, key, map) => {
        options.set(key, value);
      });
    }

    //        console.log('name=' + request.parName + ', value=' + request.parValue);
    //        opt[request.parName] = request.parValue;

    console.log(tabs[tabId]);
  }

  if ('get-toolbox-options' === request.type) {
    let obj = {};
    let modes = tabs[tabId];
    if (modes) {
      let mode = modes.get("current_mode");
      let selectedMode = null;
      if (request.mode) {
        if (mode != request.mode) {
          modes.set("current_mode", request.mode);
        }
        selectedMode = request.mode;
      } else {
        selectedMode = mode;
      }
      obj["current_mode"] = selectedMode;
      let options = modes.get(selectedMode);
      options.forEach((value, key, map) => {
        obj[key] = value;
      });
    }
    sendResponse(obj);
  }

});

function objToMap(obj) {
  let strMap = new Map();
  for (let k of Object.keys(obj)) {
    strMap.set(k, obj[k]);
  }
  return strMap;
}


function setDefaultTabCong(tabId) {
  let modes = new Map([
    ["minimized", {}],
    ["maximized", {}],
    ["current_mode", "maximized"]
  ]);
  tabs[tabId] = modes;
  modes.forEach((value, key, map) => {
    if (key != "current_mode") {
      map.set(key, new Map([
        ["wasOpenedToolbox", false]
      ]));
    }
  });
  return modes;
}
