var current_mode = null;
chrome.storage.local.get(['selectedText', 'hostUrl', 'hideToolBoxHandler', 'captiJSessionId',
        'startPlayback', 'isMaximized', 'dictObj', 'linkUrl'
    ],
    function(result) {
        var message = null;
        if (!result.isMaximized) {
            message = {
                type: 'get-toolbox-options'
            };
        } else {
            message = {
                type: 'get-toolbox-options',
                mode: 'maximized'
            };
        }
        chrome.runtime.sendMessage(message, function(resp) {
            console.log('resp=' + JSON.stringify(resp));
            initGUI(result, resp);
        });
    }
);

function initGUI(result, resp) {
    var captiToolboxContainer = document.getElementById('capti-toolbox');

    current_mode = resp ? resp.current_mode : "maximized";

    if (captiToolboxContainer && 0 == captiToolboxContainer.children.length) {
        Window.jSessionIdInTab = null;
        Window.jSessionIdInTab = result.captiJSessionId;

        eval(result.hideToolBoxHandler);

        captiToolboxContainer.classList.add("capti-toolbox-loading");

        spinnerElement = document.createElement('div');
        spinnerElement.id = 'capti-spinner-el';
        captiToolboxContainer.appendChild(spinnerElement);

        dragElement = document.createElement('div');
        dragElement.id = 'header-drag-element';
        captiToolboxContainer.appendChild(dragElement);

        dragOuterElement = document.createElement('div');
        dragOuterElement.id = 'header-drag-element-outer';
        captiToolboxContainer.appendChild(dragOuterElement);

        dragElements = [dragElement, dragOuterElement];
        dragElements.forEach(function(drEl) {
            initDragADrop(drEl, captiToolboxContainer, function(drag, e) {
                drag.x = e.screenX;
                drag.y = e.screenY;


                var diffw = drag.x - drag.oldx;
                var diffh = drag.y - drag.oldy;

                if (drag.drag) {
                    drag.dialog.removeAttribute('style');
                    drag.dialog.style.left = drag.left + diffw + 'px';
                    drag.dialog.style.height = drag.height + 'px';
                    var calcTop = drag.top + diffh;
                    if ( calcTop > 0 ) {
                        drag.dialog.style.top = calcTop + 'px';
                    } else {
                        drag.dialog.style.top = '5px';
                    }
                    e.preventDefault();
                }
            });
        });

        iframe = document.createElement('iframe');
        iframe.id = 'capti-frame-id';
        iframe.name = 'capti-iframe';
        iframe.className = "capti-frame";
        iframe.src = result.hostUrl + '/ExtensionServlet';
        iframe.setAttribute("scrolling", "no");
        iframe.setAttribute("allow", "autoplay");
        captiToolboxContainer.appendChild(iframe);


        h_sizer_el = document.createElement('div');
        h_sizer_el.id = "capti-h-size-el";
        captiToolboxContainer.appendChild(h_sizer_el);

        initDragADrop(h_sizer_el, captiToolboxContainer, function(drag, e) {
            drag.y = e.screenY;

            var diffh = drag.y - drag.oldy;

            if (drag.drag) {
                s_height = drag.height + diffh;
                if (s_height > 240) {
                    drag.dialog.style.height = s_height + 'px';
                }
                e.preventDefault();
            }
        });

        // iform.submit();

        console.log("initGUI.init");

        bindEvent(window, 'message', messageHandler);

        setToolboxOptions(resp);

    } else {
        sendMessageToCaptiApp();
    }


    function closeToolbox() {
        onHideClickHandler();
        sendCloseToolboxMessage();
    }

    function sendCloseToolboxMessage() {
        if (current_mode !== undefined) {
            chrome.runtime.sendMessage({
                type: 'option-toolbox',
                toolbox_options: mapToObj(new Map([
                    ["wasOpenedToolbox", false]
                ])),
                mode: current_mode
            });
        }
    }


    function sendMessageToCaptiApp() {
        frameElem = document.getElementById('capti-frame-id');
        if (frameElem) {
            console.log('isMinimizedMode=' + isMinimizedMode() + ', current_mode=' + current_mode);
            var frameDocument = (frameElem.contentWindow || frameElem.contentDocument);
            if (result.dictObj) {
                frameDocument.postMessage({
                    type: "*capti-msg-in-dict*",
                    dictObj: result.dictObj.definObj,
                    translateText: result.dictObj.translateText,
                    isDefinition: result.dictObj.isDefinition
                }, "*");
            } else if (result.linkUrl) {
                console.log('res_link = ' + result.linkUrl);
                frameDocument.postMessage({
                    type: "*capti-msg-in-url*",
                    importUrl: result.linkUrl
                }, '*');
            } else {
                frameDocument.postMessage({
                    type: "*capti-msg-in-text*",
                    text: result.selectedText,
                    viewMode: isMinimizedMode(),
                    playback: result.startPlayback
                }, "*");
            }
        }
    }

    function isMinimizedMode() {
        return current_mode === 'minimized';
    }

    function mapToObj(map) {
        var obj = Object.create(null);
        for (let [k, v] of map) {
            obj[k] = v;
        }
        return obj;
    }

    function initDragADrop(drEl, captiToolboxContainer, moveCallback) {
        var drag = new Object();
        drag.dialog = captiToolboxContainer;
        drag.dragEl = drEl;

        drag.dragEl.addEventListener('pointerdown', function(e) {
            drag.dragEl.setPointerCapture(e.pointerId);

            coord = drag.dragEl.getBoundingClientRect();
            coord_d = drag.dialog.getBoundingClientRect();

            drag.top = parseInt(coord.top);
            drag.left = parseInt(coord.left);
            drag.height = parseInt(coord_d.height);
            drag.oldx = e.screenX;
            drag.oldy = e.screenY;
            drag.drag = true;
        });

        drag.dragEl.addEventListener('dragstart', function(e) {
            return false;
        });


        drag.dragEl.addEventListener('pointerout', function(e) {
            if (drag.dragEl.setPointerCapture && drag.drag) {
                stopDrag(e);
            }
        });

        drag.dragEl.addEventListener('pointerup', function(e) {
            stopDrag(e);
        });

        document.addEventListener('mousemove', function(e) {
            moveCallback(drag, e);
        });

        function stopDrag(e) {
            drag.dragEl.releasePointerCapture(e.pointerId);
            drag.drag = false;

            var valueMap = new Map();
            if (drag.dialog.style.height) {
                valueMap.set("toolbox_height", drag.dialog.style.height);
            }

            valueMap.set("toolbox_top", drag.dialog.style.top);
            valueMap.set("toolbox_left", drag.dialog.style.left);
            valueMap.set("wasOpenedToolbox", true);

            chrome.runtime.sendMessage({
                type: 'option-toolbox',
                toolbox_options: mapToObj(valueMap),
                mode: current_mode
            });

            setFocusOnGDEl();
        }
    }

    function bindEvent(element, eventName, eventHandler) {
        if (element.addEventListener) {
            element.addEventListener(eventName, eventHandler, true);
        } else if (element.attachEvent) {
            element.attachEvent('on' + eventName, eventHandler);
        }
    }

    function makeParam(name, val) {
        inp = document.createElement('input');
        inp.setAttribute('type', 'hidden');
        inp.setAttribute('name', name);
        inp.setAttribute('value', val);
        return inp;
    };

    function setFocusOnGDEl() {
        var elements = document.getElementsByClassName('docs-texteventtarget-iframe');
        if (elements && 0 != elements.length) {
            elements[0].focus();
        }
    }

    function showHideResizeEl(condition) {
        var capHSizeEl = document.getElementById('capti-h-size-el');
        if (condition) {
            capHSizeEl.classList.add("capti-change-h-size");
        } else {
            capHSizeEl.classList.remove("capti-change-h-size");
        }
    }

    function changeCurrentMode(condition) {
        current_mode = condition ? 'maximized' : 'minimized';
        var valueMap = new Map();
        valueMap.set("wasOpenedToolbox", true);

        chrome.runtime.sendMessage({
            type: 'option-toolbox',
            toolbox_options: mapToObj(valueMap),
            mode: current_mode
        });

    }

    function setToolboxOptions(options) {
        var captiToolbox = document.getElementById('capti-toolbox');
        captiToolbox.removeAttribute('style');
        if (options) {
            if (options.toolbox_top) {
                captiToolbox.style.top = options.toolbox_top;
            } else {
                captiToolbox.style.top = '10px';
            }

            if (options.toolbox_left) {
                captiToolbox.style.left = options.toolbox_left;
            } else {
                captiToolbox.style.left = '100%';
            }

            if (options.toolbox_height) {
                captiToolbox.style.height = options.toolbox_height;
            }

            if ( '100%' == captiToolbox.style.left ) {
                captiToolbox.style.marginLeft = 'auto';
                captiToolbox.style.right = '10px';
            }

        }
    }

    function addRemoveClassForRoot(className, command, endStr) {
        var captiFrame = document.getElementById('capti-frame-id');
        var captidiv = document.getElementById('capti-toolbox');
        if (command.endsWith(endStr)) {
            captiFrame.classList.add(className);
            captidiv.classList.add(className);
        } else {
            captiFrame.classList.remove(className);
            captidiv.classList.remove(className);
        }
    }

    function offLoadingMode() {
        var capToolboxContainer = document.getElementById('capti-toolbox');
        captiToolboxContainer.classList.remove("capti-toolbox-loading");

        var spinnerEl = document.getElementById('capti-spinner-el');
        if (null != spinnerEl) {
            spinnerEl.remove();
        }
    }


    function messageHandler(event) {
        console.log(event);

        switch (event.data) {

            case '*capti-msg-out-load*':
                sendMessageToCaptiApp();
                break;

            case '*capti-msg-out-is-ready*':
                var capToolboxContainer = document.getElementById('capti-toolbox');
                capToolboxContainer.classList.remove("capti-toolbox-close");
                capToolboxContainer.classList.add("capti-toolbox-open");

                offLoadingMode();

                var capHSizeEl = document.getElementById('capti-h-size-el');
                if (!isMinimizedMode()) {
                    capHSizeEl.classList.add("capti-change-h-size");
                }

                addRemoveClassForRoot('minimized-toolbox', isMinimizedMode() ? 'minimize*' : '', 'minimize*');

                var valueMap = new Map();
                if (capToolboxContainer.style.height) {
                    valueMap.set("toolbox_height", capToolboxContainer.style.height);
                }

                valueMap.set("toolbox_top", capToolboxContainer.style.top);
                valueMap.set("toolbox_left", capToolboxContainer.style.left);
                valueMap.set("wasOpenedToolbox", true);

                chrome.runtime.sendMessage({
                    type: 'option-toolbox',
                    toolbox_options: mapToObj(valueMap),
                    mode: current_mode
                });
                break;

            case '*capti-msg-out-logout*':
                chrome.runtime.sendMessage({
                    logoutMessage: event.data
                });
                break;

            case '*capti-msg-out-close-toolbox*':
                closeToolbox();
                break;

            case 'capti-remove-message-handler':
                sendCloseToolboxMessage();
                window.removeEventListener('message', messageHandler, true);
                break;

            case '*capti-msg-out-text-option-open*':
            case '*capti-msg-out-text-option-close*':
                addRemoveClassForRoot('text-option-dialog', event.data, 'open*');
                break;

            case '*capti-msg-out-import-doc-open*':
            case '*capti-msg-out-import-doc-close*':
                addRemoveClassForRoot('import-doc-dialog', event.data, 'open*');
                break;

            case '*capti-msg-out-toolbox-minimize*':
            case '*capti-msg-out-toolbox-maximize*':
                addRemoveClassForRoot('minimized-toolbox', event.data, 'minimize*');
                showHideResizeEl(event.data.endsWith('maximize*'));
                changeCurrentMode(event.data.endsWith('maximize*'));
                chrome.runtime.sendMessage({
                    type: 'get-toolbox-options'
                }, function(resp) {
                    setToolboxOptions(resp);
                });
                break;

            case '*capti-msg-out-err-open*':
            case '*capti-msg-out-err-close*':
                addRemoveClassForRoot('error-showed', event.data, 'open*');
                break;

            case '*capti-msg-out-dic-open*':
            case '*capti-msg-out-dict-close*':
                addRemoveClassForRoot('dict-open', event.data, 'open*');
                offLoadingMode();
                break;

            case '*capti-msg-out-play-open*':
            case '*capti-msg-out-play-close*':
                addRemoveClassForRoot('play-showed', event.data, 'open*');
                showHideResizeEl(event.data.endsWith('close*'));
                break;

            case '*capti-msg-out-url-open*':
            case '*capti-msg-out-url-close*':
                addRemoveClassForRoot('url-dialog-open', event.data, 'open*');
                offLoadingMode();
                break;
        }
    }

}
