var bg = chrome.extension.getBackgroundPage();
var blocked_tab_id = null;

chrome.runtime.onMessage.addListener(
  function(request, sender, sendResponse) {
    if (request.logoutMessage && request.logoutMessage === "*capti-msg-out-logout*") {
      hideToolBox();
      showToolBox(null, false);
    }

    if (request.message && (request.message === '*google-menu-item-action*' &&
        sender.url.startsWith('https://docs.google.com')) || request.message == '*ms-menu-item-action*') {
      bg.executeShowToolbox(false, true);
    }

    if (request.dictionary_extractor && request.dictionary_obj) {
      if (null != request.dictionary_obj) {
        var dic_obj = request.dictionary_obj;
        if (dic_obj.isDefinition && null == dic_obj.definObj) {
          showErrorNotification("definitionErrMessage");
          return;
        }

        if (null == dic_obj.translateText || dic_obj.translateText == '') {
          showErrorNotification("dictionaryErrMessage");
          return;
        }
        showBaseToolBox({
          startPlayback: false,
          dictionaryObj: dic_obj
        });
      } else {
        console.log('result is absent');
      }
    }

    if (request.external_menu_message) {

      if (request.external_menu_message.startsWith('*external-menu-item-dict*') && undefined != request.isDefinition) {
        console.log(Object.values(request.selectionObj));
        showDictionaryDialog(request.isDefinition, request.selectionObj);
      }

      if (request.external_menu_message.startsWith('*external-menu-item-view-select*:')) {
        var text = request.external_menu_message.replace('*external-menu-item-view-select*:', '').trim();
        var isPlaying = (request.is_playing ? request.is_playing : false);
        var isMaximized = (!isPlaying ? true : null);
        showBaseToolBox({
          selectedText: text,
          startPlayback: isPlaying,
          isMaximized: isMaximized
        });
      }
    }

    if (undefined !== request.startedTest) {
      if ( request.startedTest ) {
        hideToolBox();
        blocked_tab_id = sender.tab.id;
      } else {
        blocked_tab_id = null;
      }
    }

  });

chrome.tabs.onActivated.addListener(function(activeInfo) {
  getToolboxState(function(isShowed) {
    if (null != isShowed && undefined != isShowed && isShowed) {
      bg.sendRequestWorkspacePage(function() {
        chrome.tabs.executeScript(activeInfo.tabId, {
          code: 'Window.jSessionIdInTab;'
        }, function(results) {
          bg.getJSessionId(function(captiSessionId) {
            var jSessionIdInTab = results[0];
            console.log("jSessionIdInTab = " + jSessionIdInTab);
            if (captiSessionId != jSessionIdInTab) {
              hideToolBox();
            }
          });
        });
      });
    }
  });
});

function showToolBox(selectionText, startPlayback) {
  showBaseToolBox({
    selectedText: selectionText,
    startPlayback: startPlayback
  });
}

function showBaseToolBox(params) {
  chrome.tabs.query({
    active: true,
    currentWindow: true
  }, function(tabs) {
    var selectedTab = tabs[0];
    if (null == blocked_tab_id || blocked_tab_id !== selectedTab.id) {
      bg.checkLogin(function() {
        bg.getJSessionId(function(sessionId) {
          chrome.storage.local.set({
                selectedText: checkParameter(params.selectedText),
                hostUrl: bg.hostUrl,
                hideToolBoxHandler: onHideClickHandler.toString(),
                captiJSessionId: sessionId,
                startPlayback: checkParameter(params.startPlayback),
                isMaximized: checkParameter(params.isMaximized),
                dictObj: checkParameter(params.dictionaryObj),
                linkUrl: checkParameter(params.url)
              },
              function() {
                chrome.tabs.executeScript(selectedTab.id, {
                  file: 'toolbox_dialog.js'
                });
              });
        });
      });
    }
  });
}

function checkParameter(parameter) {
  return (undefined != parameter ? parameter : null);
}


function hideToolBox() {
  chrome.tabs.executeScript(null, {
    code: onHideClickHandler.toString() + ' onHideClickHandler()'
  });
}

function getToolboxState(callback) {
  chrome.tabs.query({
    active: true,
    currentWindow: true
  }, function(tabs) {
    var selectedTab = tabs[0];
    if (!selectedTab.url.startsWith("chrome")) {
      chrome.tabs.executeScript(selectedTab.id, {
          code: "var captiToolboxContainer = document.getElementById('" + bg.captiToolboxId + "'); if( captiToolboxContainer ) { if(captiToolboxContainer.children && 0 != captiToolboxContainer.children.length) {true} else {false} } else {null}"
        },
        function(results) {
          if (null != results && null != results[0]) {
            callback(results[0]);
          } else {
            callback(null);
          }
        }
      );
    } else {
      callback(undefined);
    }
  });
}


function onHideClickHandler() {
  var captiToolboxContainer = document.getElementById("capti-toolbox");
  captiToolboxContainer.className = "capti-toolbox-close";
  while (captiToolboxContainer.firstChild) {
    captiToolboxContainer.removeChild(captiToolboxContainer.firstChild);
  }

  var message = "capti-remove-message-handler";
  var origin = window.location.protocol + "//" + window.location.host;
  var lastEventId = "";
  var messageEvent = new MessageEvent("message", {data:message, origin:origin, lastEventId:lastEventId});
  window.dispatchEvent(messageEvent);
}

function showDictionaryDialog(isDefinition, selectionObj) {
  console.log(selectionObj);
  chrome.storage.local.set({
    isDefinition: isDefinition,
    selection: selectionObj
  }, function() {
    chrome.tabs.executeScript(null, {
      file: 'dictionary_extractor.js'
    });
  });
}

function showImportLinkDialog(url) {
  showBaseToolBox({
    startPlayback: false,
    url: url
  });
}

function showErrorNotification(errorMessage) {
  chrome.notifications.create({
    type: "basic",
    title: bg.appName,
    iconUrl: "assets/capti-36.png",
    message: chrome.i18n.getMessage(errorMessage)
  });
}

function showErrorStateNotification(variable) {
  if (undefined === variable) {
    bg.showErrorNotification("cannotShowToolboxErrorMessage");
  } else {
    bg.showErrorNotification("cannotShowToolboxErrorMessage");
  }
}
